﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Csharp_review4_event
{
    // (stringを一つ引数にとって、intを返すメソッドの)デリゲートを定義します。
    public delegate int MyDelegate(string arg);

    class Program
    {
        static void Main(string[] args)
        {
            // プロバイダクラスのオブジェクトを作って、
            MethodProvider obj = new MethodProvider();

            // プロバイダのメソッドを引数に、実行クラスのオブジェクトを作って、
            DriveClass drvObj = new DriveClass(obj.ClassMethod1);

            // 実行クラスを動かす
            drvObj.Run();

            // 登録されているメソッドを書き換えます。
            //drvObj.MyEvent= staticClassMethod; --> これはコンパイルエラー

            // 実行クラスの外からデリゲートを実行してみる
            //drvObj.MyEvent("From Main");  --> これもコンパイルエラー

            Console.WriteLine("===Add event handler===");
            drvObj.MyEvent += staticClassMethod;    // += ならOK

            // イベントを発火させられるのはイベントを持ったクラスのみ
            drvObj.Run();

            Console.ReadLine();
        }

        static int staticClassMethod(string arg)
        {
            Console.WriteLine("staticClassMethod started. arg = [{0}]", arg);
            return 0;
        }
    }

    // メソッドを提供するクラス
    public class MethodProvider
    {
        public int ClassMethod1(string arg)
        {
            Console.WriteLine("ClassMethod1 started. arg = [{0}]", arg);
            return 0;
        }
    }

    // 実行者クラス
    public class DriveClass
    {
        public event MyDelegate MyEvent;  // delegateをイベントとして定義します。

        public DriveClass(MyDelegate func)
        {
            MyEvent += func;
        }

        public int Run()
        {
            int ret = 0;

            if (MyEvent != null)
            {
                ret = MyEvent("From DriveClass.Run()");
            }

            return ret;
        }
    }
}
